## MH Avoidable data preparation
## Read in data, winsorizing, prepare variables

# Peter Martin



## Packages
source("Packages.R")

#functions
num_na = function(x) {sum(is.na(x))}
centre_var = function(x) {x-mean(x, na.rm = TRUE)}
stand_var = function(x) {(x-mean(x, na.rm = TRUE))/sd(x, na.rm = TRUE)}


#########################
####### Data ############
########################


# Read in data
AA = read.csv("MH_Avoidable.csv", header = TRUE)

# Make sure rural-urban is a categorical variable
AA = within(AA, {
  RUC11CD = as.factor(RUC11CD)
})


#########################
##### Winsorize #########
#########################



### Winsorisation reloaded

# All variables 

to_wins = c("nomhsdssaar_av1618", "nonorgsaar_av1618", "nonorgphyssaar_av1618", "mhsdsphyssaar_av1618","nonorgmentsaar_av1618","orgmentsaar_av1618", "orgphyssaar_av1618", "nomhphyssaar_av1618",
            "prcnt_nonwhite",
            "income_ave_score",
            "employ_ave_score",
            "educ_ave_score",
            "barriers_ave_score",
            "living_ave_score",
            "crime_ave_score",
            "o75_prcnt",
            "qof_hyp",
            "qof_copd",
            "qof_dm",
            "qof_smi",
            "qof_dep",
            "prnct_mhsu_mha",
            "pcnt_popo18_mhsds",
            "qof1617_totachieverate_hyp",
            "qof1617_totachieverate_copd",
            "qof1617_totachieverate_dm",
            "qof1617_totachievrate_mh",
            "qof1617_totachievrate_dep",
            "iapt_firsttx6wks_av1718",
            "iapt_accessrate1718",
            "prcnt_gpsinglehand",
            "gp_p100000",
            "prcnt_48hrsseegpnurse",
            "prcnt_noappt",
            "admission_gp_pcnt",
            "aeattendrate_av1618",
            "mhactualspendccgalloc_1718",
            "prcnt_mhsu_pop_CPA",
            "prnct_CPA_employ",
            "prcnt_acutadmit_crht_av1718",
            "nhs111_ambdesp_rate", "nhs111_rec_ae_rate", "f2fresp_nottyp1_2_ave_apr17_mar1", "prcnt_coreccg_aeliaisactual_1718",
            "prcnt_crhttactualspend_1718", "prcnt_eipactualspend_1718", "prcnt_cpa_7days_av1718")

forwins = AA[,to_wins]
m = dim(forwins)[2]

sort(winsor(forwins[,1], trim = 0.025, na.rm = TRUE))


xywins = matrix(rep(NA, (207*m)),nrow=207, ncol=m)
for (i in 1:m) {
  xywins[,i] = winsor(forwins[,i], trim = 0.025, na.rm = TRUE)
}

colnames(xywins)=names(forwins)

xywins = data.frame(xywins)



################################################
####### PCA of qof variables from Block 3 ######
################################################

summary(xywins[,c("qof1617_totachieverate_hyp",
                  "qof1617_totachieverate_copd",
                  "qof1617_totachieverate_dm",
                  "qof1617_totachievrate_mh",
                  "qof1617_totachievrate_dep")])
qofcormat = round(cor(xywins[,c("qof1617_totachieverate_hyp",
                    "qof1617_totachieverate_copd",
                    "qof1617_totachieverate_dm",
                    "qof1617_totachievrate_mh",
                    "qof1617_totachievrate_dep")]), 2)
round(cor(AA[,c("qof1617_totachieverate_hyp",
                "qof1617_totachieverate_copd",
                "qof1617_totachieverate_dm",
                "qof1617_totachievrate_mh",
                "qof1617_totachievrate_dep")]),2)
write.csv(qofcormat, "Correlation matrix of winsorised QOF achievement variables.csv", row.names = FALSE)

pcaqof = prcomp(xywins[,c("qof1617_totachieverate_hyp",
                          "qof1617_totachieverate_copd",
                          "qof1617_totachieverate_dm",
                          "qof1617_totachievrate_mh",
                          "qof1617_totachievrate_dep")], scale.= TRUE)
summary(pcaqof) # First component summarises 83 % of the variance

pcacoefs = pcaqof$rotation
sqevs = pcaqof$sdev

pcaloadings = cbind(pcacoefs[,1]*sqevs[1],
                     pcacoefs[,2]*sqevs[2],
                     pcacoefs[,3]*sqevs[3],
                     pcacoefs[,4]*sqevs[4],
                     pcacoefs[,5]*sqevs[5])



## Save first component as variable (and change direction so that higher value -> higher achievement rate)
xywins$QOFpc = (-1)*pcaqof$x[,1]


############################################
############ Look at IMD variables #########
############################################

summary(xywins[,c("income_ave_score",
                  "employ_ave_score",
                  "educ_ave_score",
                  "barriers_ave_score",
                  "living_ave_score",
                  "crime_ave_score")])
round(cor(xywins[,c("income_ave_score",
                    "employ_ave_score",
                    "educ_ave_score",
                    "barriers_ave_score",
                    "living_ave_score",
                    "crime_ave_score")]), 2)
round(cor(AA[,c("income_ave_score",
                "employ_ave_score",
                "educ_ave_score",
                "barriers_ave_score",
                "living_ave_score",
                "crime_ave_score")]),2)




############################################
####### Further data preparation ###########
############################################

# Standardise all variables
xystand = apply(xywins, 2, stand_var)


## All variables (including predictors with missing values)
xyall = data.frame(cbind(xystand, AA[,c("RUC11CD", "north_south")]))


## Remove the indicators for the PCA
pca_vars = c("qof1617_totachieverate_hyp","qof1617_totachieverate_copd","qof1617_totachieverate_dm","qof1617_totachievrate_mh", "qof1617_totachievrate_dep")
xyfin = xyall[ , !(names(xyall) %in% pca_vars)]


#Recode categorical variables into dummies and delete original variables
xyfin2 = within(xyfin, {
  RUC11CD456 = ifelse(RUC11CD == "4" | RUC11CD == "5" | RUC11CD == "6", 1, 0)
  north = ifelse(north_south == "north", 1, 0)
  south = ifelse(north_south == "south", 1, 0)
})

cat_vars = c("north_south", "RUC11CD")
xyfin2 = xyfin2[ , !(names(xyfin2) %in% cat_vars)]

