### Project: Avoidable hospital admissions among MH service users
### Lead: Charlotte Woodhead
### Analysis of CCG-level data set

##Penalised regression to examine predictors of avoidable admission rate

# Analysis by Peter Martin

# predicting avoidable standardised admission rates for MHSDS patients: admissions for physical problems

#Data & preparation 
source("MH avoidable - multiple imputation.R")

source("MH lasso prep functions.R")
source("MH bootstrap functions.R")

MHP_df1 = MHP_df(xymice1)
MHP_df2 = MHP_df(xymice2)
MHP_df3 = MHP_df(xymice3)
MHP_df4 = MHP_df(xymice4)
MHP_df5 = MHP_df(xymice5)

MHP_df1 = as.data.frame(apply(MHP_df1, 2, stand_var)) # this includes 'standardising' the dummies for the purpose of calculating the importance score later on ...
MHP_df2 = as.data.frame(apply(MHP_df2, 2, stand_var))
MHP_df3 = as.data.frame(apply(MHP_df3, 2, stand_var))
MHP_df4 = as.data.frame(apply(MHP_df4, 2, stand_var))
MHP_df5 = as.data.frame(apply(MHP_df5, 2, stand_var))

head(MHP_df1)


#MHP: non-organic MH, avoidable admissions for physical problems


#################################
#### Bootstrap ##################
#################################

RNGkind(sample.kind = "Rounding")
set.seed(1824)
 
MHP_df1_B <- bootsamp_list(MHP_df1, 40)
MHP_df2_B <- bootsamp_list(MHP_df2, 40)
MHP_df3_B <- bootsamp_list(MHP_df3, 40)
MHP_df4_B <- bootsamp_list(MHP_df4, 40)
MHP_df5_B <- bootsamp_list(MHP_df5, 40)


MHP_df_B = c(MHP_df1_B, MHP_df2_B, MHP_df3_B, MHP_df4_B, MHP_df5_B) # 200 bootstrap samples collated in a list
rm(MHP_df1_B, MHP_df2_B, MHP_df3_B, MHP_df4_B, MHP_df5_B)



##################################
########### Lasso ################
##################################

nvars = dim(MHP_df1)[2] # 39 variables

# Set up
N = 207
NB = length(MHP_df_B)
grid = 10^seq(3, -2, length = 1000)
ttx <- model.matrix(nonorgphyssaar_av1618~., MHP_df_B[[1]])
tty <- MHP_df_B[[1]]$nonorgphyssaar_av1618
lasso.out <- glmnet(ttx, tty, alpha=1, lambda = grid)
lasso.coef <- predict(lasso.out, type="coefficients", s=1)
coef_names = lasso.coef@Dimnames[[1]][-1]

EST = data.frame(cbind(coef_names, matrix(c(rep(NA, (nvars*(NB)))), nrow = nvars)))
names(EST) = c("IV", paste("Est", 1:NB, sep = ""))



## Data frames
system.time(
for (i in 1:NB) {
  ttx <- model.matrix(nonorgphyssaar_av1618~., MHP_df_B[[i]])
  tty <- MHP_df_B[[i]]$nonorgphyssaar_av1618

train <- sample(1:N, N/2)
test <- (-train)

ytest <- tty[test]
ytrain <- tty[train]
xtest <- ttx[test,]
xtrain <- ttx[train,]

lasso.mod <- glmnet(xtrain, ytrain, alpha = 1, lambda = grid)
cv.out <- cv.glmnet(xtrain, ytrain, alpha = 1)
bestlam <- cv.out$lambda.min
lasso.out <- glmnet(ttx, tty, alpha=1, lambda = grid)
lasso.coef <- predict(lasso.out, type="coefficients", s=bestlam)

selected_vars = names(lasso.coef[,1])[lasso.coef[,1]!=0]
selected_vars = selected_vars[-1]

olsx = ttx[,selected_vars]
olsxy = data.frame(cbind(tty, olsx))
m_ols = lm(tty ~ ., data = olsxy)
coef_ols = data.frame(IV = names(coef(m_ols)), est = coef(m_ols))

df_aux = merge(EST[,1:2], coef_ols, by = "IV", all = TRUE)
df_aux$est[is.na(df_aux$est)] = 0
df_aux$IV = factor(df_aux$IV, levels = coef_names)
df_aux = df_aux[order(df_aux$IV),]
lasso.coef = df_aux$est
EST[,(i+1)] <- as.numeric(lasso.coef)
  }
) 


MHP_importance = data.frame(IV = EST$IV,
                             imp = abs(apply(EST[,2:(NB+1)], 1, mean)))[-1,]


########################################
####### Stage 3: Sampling variables ####
########################################


var_selection = list()

for (i in 1:NB) {
  var_selection[[i]] = c("nonorgphyssaar_av1618",as.character(impsamp(MHP_importance, (nvars-1))))
}


###Bootstrap lasso with variables selected at random

MHP_df_BB = MHP_df_B

for (i in 1:NB) {
  MHP_df_BB[[i]] = MHP_df_BB[[i]][,var_selection[[i]]]
}

EST3 = data.frame(cbind(coef_names, matrix(c(rep(NA, (nvars*(NB)))), nrow = nvars)))
names(EST3) = c("IV", paste("Est", 1:NB, sep = ""))

system.time(
  for (i in 1:NB) {
    ttx <- model.matrix(nonorgphyssaar_av1618~., MHP_df_BB[[i]])
    tty <- MHP_df_BB[[i]]$nonorgphyssaar_av1618
    
    train <- sample(1:N, N/2)
    test <- (-train)
    
    ytest <- tty[test]
    ytrain <- tty[train]
    xtest <- ttx[test,]
    xtrain <- ttx[train,]
    
    lasso.mod <- glmnet(xtrain, ytrain, alpha = 1, lambda = grid)
    cv.out <- cv.glmnet(xtrain, ytrain, alpha = 1)
    bestlam <- cv.out$lambda.min
    lasso.out <- glmnet(ttx, tty, alpha=1, lambda = grid)
    lasso.coef <- predict(lasso.out, type="coefficients", s=bestlam)
    
    selected_vars = names(lasso.coef[,1])[lasso.coef[,1]!=0]
    selected_vars = selected_vars[-1]
    
    olsx = ttx[,selected_vars]
    olsxy = data.frame(cbind(tty, olsx))
    m_ols = lm(tty ~ ., data = olsxy)
    coef_ols = data.frame(IV = names(coef(m_ols)), est = coef(m_ols))
    
    df_aux = merge(EST3[,1:2], coef_ols, by = "IV", all = TRUE)
    df_aux$est[is.na(df_aux$est)] = 0
    df_aux$IV = factor(df_aux$IV, levels = coef_names)
    df_aux = df_aux[order(df_aux$IV),]
    lasso.coef = df_aux$est
    EST3[,(i+1)] <- as.numeric(lasso.coef)
  }
)


EST3$beta_initial = apply(EST3[,2:(NB+1)], 1, mean)
EST3$selection_prob = apply(EST3[,2:(NB+1)], 1, selprob)


EST3$beta_lower = apply(EST3[,2:(NB+1)], 1, p025)
EST3$beta_upper = apply(EST3[,2:(NB+1)], 1, p975)


EST3[,c("IV", "beta_initial", "beta_lower", "beta_upper","selection_prob")]


####################################
### STAGE 4: Stability Selection ###
####################################

## setup

p_thresh = (20:180)/200 # Sensible values of threshold for selection probability
lp = length(p_thresh)
STAB = data.frame(cbind(p_thresh, matrix(c(rep(NA, (lp*(NB)))), nrow = lp)))
names(STAB) = c("threshold", paste("MSQ", 1:NB, sep = ""))

ESTPROBS = EST3[-1,c("IV", "selection_prob")]


system.time(
  for (i in 1:NB) {
    ttdf <- MHP_df_B[[i]]
    
    train <- sample(1:N, N/2)
    test <- (-train)
    
    df.test <- ttdf[test,]
    df.train <- ttdf[train,]
    q = dim(df.test)[2]
    
    for (j in 1:lp) {
      selvars = as.character(ESTPROBS[ESTPROBS$selection_prob>=p_thresh[j],]$IV)
      df.train = df.train[,c("nonorgphyssaar_av1618",selvars)]
      lm.train = lm(nonorgphyssaar_av1618~., df.train)
      pred = predict(lm.train, newdata = df.test)
      mspe = sum((df.test$nonorgphyssaar_av1618 - pred)^2)/q
      STAB[j, i+1] = mspe
    }
  }
) 

STAB$mspe_est = apply(STAB[,2:(NB+1)], 1, mean)
STAB$mspe_se = apply(STAB[,2:(NB+1)], 1, sd)/sqrt(NB-1)

STAB[,c("threshold", "mspe_est", "mspe_se")]

ggplot(STAB, aes(x = threshold, y = mspe_est)) + 
  geom_errorbar(aes(x = threshold, ymin = mspe_est - mspe_se, ymax = mspe_est + mspe_se), colour = "grey", lwd = 0.5) +
  geom_point() +
  scale_x_continuous(breaks = (2:18)/20) +
  theme_classic()
                       
# prediction error is minimised at 0.485 <= p_thresh <= 0.745
# the most parsimonious model whose mspe is less than 1 SE above that of the apparently best fitting model is selected for 0.750 <= p_thresh <= 0.785

highprob = EST3[EST3$selection_prob>0.75,c("IV", "beta_initial", "beta_lower", "beta_upper","selection_prob")]
highprob # 6 predictors

tab_MHP = highprob[-1,c("IV", "beta_initial","beta_lower", "beta_upper")]
names(tab_MHP) = c("IV", "Estimate", "lo95", "up95")
tab_MHP[,2:4] = round(tab_MHP[,2:4], 3)
tab_MHP = tab_MHP[order(abs(tab_MHP$Estimate), decreasing = TRUE),]
tab_MHP

write.csv(tab_MHP, "MHSDS Physical MIRL results table.csv", row.names = FALSE)
